"""
RDS é **acrônimo** para Rede de Dados em Saúde. Este framework/api condensa um conjunto de boas práticas para o
desenvolvimento das aplicações que compõem ou comporão a RDS do LAIS e dos parceiros que contratarem o LAIS para fazer
suas próprias RDS, a exemplo REDS-RN e RDS-ES.
"""
